;msd8x - USB Mass Storage Interface v0.95
; (C) 2005 by Brandon Wilson and Michael Vincent. All rights reserved. 
;Portions in msd8x.asm, util.asm, usb8x.inc, equates.inc (C) Dan Englender.

;v0.95 Changelog
;MirageOS BASIC programs can now be executed
;+/- buttons can be used to scroll up/down one page at a time in file explorer
;usb8x MSD caching is used if available to speed up file browsing
;Text files can be viewed by pressing ENTER
;Text files can be scrolled through by holding DOWN and viewed from the beginning with Y=
;Source code completely reorganized
;Many new vendor/product ID combinations added for auto-launch
;Files and folders can now be renamed
;Directories can be entered with either RIGHT or ENTER, and the previous directory can be accessed with LEFT
;More descriptive error strings added

 include "settings.inc" ;Specific settings for this application
 NOLIST
 include "ti83plus.inc"
 LIST
 include "equates.inc" ;Equates and macros to be used
 include "header.asm" ;Application header
 include "usb8x.inc" ;usb8x equates

 SEGMENT MAIN
 GLOBALS ON
 
 EXTERN fastCopy,DialogBox,DispHexHL,IGetKey,IPutS,DispHexA,DispA,vputstring,FunkyLine,VEraseEOL,Traverse_symTable
 EXTERN fixVarNames,cmpstrb,putsb,Explore_QuickSort,Divide32,Explore_Add,Explore_Copy,Explore_Del,Explore_Ren
 EXTERN Explore_New,Explore_Disp8XI,Explore_RunProg,deleteTempProgram,USBError,USBErrorNoDrv,USBErrorVersion
 EXTERN ViewFile,IPutS11

;-----------------------------------------------------------------------------
InitProg:
       res apdAble,(iy+apdFlags)
       xor a
       out (28h),a ;screws with stuff, I don't know how/why
	inc a
	out (20h),a
       call CallInit
	ld hl,USBDriverBuf
	U_CALL DriverInit
	ld hl,appBackUpScreen
	ld de,appBackUpScreen+256
	U_CALL MSD_Initialize
       jr nc,$F
       U_CALL HostKill
       U_CALL DriverKill
       B_CALL ClrLCDFull
       B_CALL HomeUp
       ld hl,sReconnect
       call IPutS
       call IGetKey
       cp kQuit
       jr z,AppExit
       cp kClear
       jr z,AppExit
       jr InitProg
$$:	U_CALL UFI_Initialize
	jr c,USBError
	U_CALL FAT_Initialize
	jr c,USBError
       jr Explore_ListDirectoryStay
CallInit:
       U_CALL_INIT DummyRet
       ret
Init:  B_CALL DelRes
       call deleteTempProgram ;just in case
       res apdAble,(iy+apdFlags)
       ld hl,sRoot
       ld de,current_dir
       ldi
       ldi
       xor a
       out (28h),a ;screws with stuff, I don't know how/why
	inc a
       out (20h),a
	call CallInit
	jr c,USBErrorNoDrv
       U_CALL GetVersion
       ld de,0*256+11
       or a
       sbc hl,de
       jr c,USBErrorVersion
	ld hl,USBDriverBuf
	U_CALL DriverInit
	ld hl,appBackUpScreen
	ld de,appBackUpScreen+256
	U_CALL MSD_Initialize
	jr c,USBError
	U_CALL UFI_Initialize
	jr c,USBError
	U_CALL FAT_Initialize
	jr c,USBError
;-----------------------------------------------------------------------------
Explore_ListDirectory:
	ld hl,0
	ld (file_start),hl
	ld (file_cur),hl
Explore_ListDirectoryStay:
       ld bc,0001h
       ld e,c
       U_CALL MSD_cacheOn
	ld hl,current_dir
	U_CALL DOS_countFilesInDir
	ld (file_count),bc
	ld hl,1170 ;max number of files per directory...(4000h/(12+2) < 4000h) (all you can fit on a RAM page)
	or a
	sbc hl,bc
	jr c,AppExit
	ld hl,current_dir
       push hl
	U_CALL FAT_getStartingCluster
       pop de
	jr nc,Explore_LD_1
       ld hl,sRoot ;path was invalid, so start over at root
       ldi
       ldi
       ld hl,0
Explore_LD_1:
	ld (dir_cluster),hl
       B_CALL ClrLCDFull
	ld hl,0303h
	ld (currow),hl
	ld hl,sIndexing
	call IPutS
	B_CALL RunIndicOn
	;Populate RAM page 87h with the names, 86h with pointers
	ld ix,8000h
	ld de,8000h
       ld bc,-1
Explore_PopulateL:
	push de
	push ix
	ld hl,(dir_cluster)
	U_CALL DOS_getNextFile
	pop ix
	pop de
	jr c,Explore_SortL
	ld a,(hl)
	cp '.'
	jr nz,$F
	inc hl
	ld a,(hl)
	dec hl
	cp '.'
	jr z,$F
	ld hl,(file_count)
	dec hl
	ld (file_count),hl
	jr Explore_PopulateL
$$:	push bc
	push hl
	push de
	ld bc,12
	ld de,OP7
	ldir
	pop de
	pop hl
	di
	ld a,86h
	out (7),a
	ld (ix),e
	ld (ix+1),d
	inc ix
	inc ix
	ld a,87h
	out (7),a
	ld hl,OP7
	ld bc,12
	ldir
	pop bc
	ld a,c
	ld (de),a
	inc de
	ld a,b
	ld (de),a
	inc de
	ld a,81h
	out (7),a
	ei
	jr Explore_PopulateL
Explore_SortL:
	ld de,3*256+3
	ld (currow),de
	ld hl,sSorting
	call IPutS
	B_CALL EraseEOL
	di
	ld hl,(file_count)
	ld a,86h
	out (7),a
	dec hl
	add hl,hl
	ld bc,8000h
	add hl,bc
	ex de,hl
	ld hl,8000h
	call Explore_QuickSort
	ld a,81h
	out (7),a
	ei
;Display main screen
       B_CALL RunIndicOff
	B_CALL ClrLCDFull
	B_CALL GrBufClr
	ld hl,0
	ld de,6*256+95
	B_CALL FillRect
       ld hl,0003h
       ld (pencol),hl
	ld hl,current_dir
	set textInverse,(iy+textFlags)
	call vputstring
	res textInverse,(iy+textFlags)
	ld bc,51*256+7
	ld de,51*256+57
	B_CALL DarkLine
	ld bc,7
	ld de,95*256+7
	B_CALL DarkLine
	ld bc,19*256+7
	ld de,19*256
	B_CALL DarkLine
	ld bc,38*256+7
	ld de,38*256
	B_CALL DarkLine
	ld bc,57*256+7
	ld de,57*256
	B_CALL DarkLine
	ld bc,76*256+7
	ld de,76*256
	B_CALL DarkLine
	ld hl,57*256+2
	ld (pencol),hl
	ld hl,sNavTable
	ld b,5
Explore_NavLoop:
	ld a,(pencol)
	ld c,a
	push bc
	call vputstring
	pop bc
	ld a,c
	add a,19
	ld (pencol),a
	djnz Explore_NavLoop
Explore_LoopBack:
	ld de,8*256
	ld (pencol),de
	ld hl,(file_start)
	ld b,7
Explore_ListFiles_Loop:
	ld de,(file_count)
	or a
       sbc hl,de
	jr z,Explore_LoopBack_NoList
	push bc
	add hl,de
	di
	push hl
	add hl,hl
	ld de,8000h
	add hl,de
	ld a,86h
	out (7),a
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ld a,87h
	out (7),a
	ld de,OP7
	ld bc,12
	ldir
	ld a,81h
	out (7),a
	ei
	ld hl,OP7
	ld de,OP1
	U_CALL FAT_nameConvertFrom11
	ld a,(OP7+11)
	and 10h
	ld a,6
	jr z,$F
	ld a,'+'
$$:	B_CALL VPutMap
	call vputstring
Explore_LFL_1:
	ld a,(pencol)
	cp 51
	jr nc,Explore_LFL_2
	ld a,' '
	B_CALL VPutMap
	jr Explore_LFL_1
Explore_LFL_2:
	xor a
	ld (pencol),a
	ld a,(penrow)
	add a,6
	ld (penrow),a
	pop hl
	push hl
	ld de,(file_cur)
	or a
	sbc hl,de
	jr nz,$F
	add hl,de
	ld de,(file_start)
	or a
	sbc hl,de
	call Explore_Rects_R
$$:	pop hl
	inc hl
	pop bc
	djnz Explore_ListFiles_Loop
Explore_LoopBack_NoList:
;Display information for the current file
	ld hl,sSize
	ld de,8*256+53
	ld (pencol),de
	call vputstring
	ld a,70
	ld (pencol),a
	di
	ld hl,(file_cur)
	ld a,86h
	out (7),a
	add hl,hl
	set 7,h
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ld a,87h
	out (7),a
	ld de,12
	add hl,de
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld a,81h
	out (7),a
	ei
	ld hl,(dir_cluster) ;BC is entry number
	U_CALL DOS_getDirEntry
       jr nc,$F
       ld hl,cur_entry
       ld (hl),0
       ld de,cur_entry+1
       ld bc,31
       jr getFileSize
$$:    ld de,cur_entry
	ld bc,32
getFileSize:
	ldir
	ld hl,cur_entry
	U_CALL DOS_getFileSize
	call Explore_DisplayFileSize
	ld hl,15*256+53
	ld (pencol),hl
	ld hl,sAttributes
	call vputstring
	ld a,70
	ld (pencol),a
	ld b,6
	ld a,(cur_entry+11)
	ld hl,Explore_Attrs-1
Explore_Attr_L:
	inc hl
	rra
	jr nc,$F
	push af
	ld a,(hl)
	or a
	jr z,Explore_Attr_L2
	B_CALL VPutMap
	ld a,' '
	B_CALL VPutMap
Explore_Attr_L2:
	pop af
$$:	djnz Explore_Attr_L
;Display byte size
	ld hl,29*256+53
	ld (pencol),hl
	ld hl,cur_entry
	U_CALL DOS_getFileSize
	call Explore_DisplayByteSize
	ld b,8
$$:	ld a,6
	B_CALL VPutMap
	djnz $B
	ld hl,36*256+53
	ld (pencol),hl
	ld hl,(cur_entry+24)
	srl h
	rr l
	ld a,l
	and 11110000b
	rrca
	rrca
	rrca
	rrca
	call DispA
	ld a,'-'
	B_CALL VPutMap
	ld hl,(cur_entry+24)
	ld a,l
	and 00011111b
	call DispA
	ld a,'-'
	B_CALL VPutMap
	ld a,h
	and 11111110b
	rrca
	cp 20
	jr c,$F
	sub 100
$$:	add a,80
	call DispA
	ld hl,43*256+53
	ld (pencol),hl
	ld hl,(cur_entry+22)
	ld a,h
	and 11111000b
	rrca
	rrca
	rrca
	call DispA
	ld a,':'
	B_CALL VPutMap
	add hl,hl
	add hl,hl
	add hl,hl
	ld a,h
	and 00111111b
	call DispA
	ld a,':'
	B_CALL VPutMap
	ld a,l
	and 11111000b
	rrca
	rrca
	rrca
	add a,a
	call DispA
;-----------------------------------------------------------------------------
Explore_DispDone:
       U_CALL MSD_cacheOff
Explore_KeyLoop:
       res apdAble,(iy+apdFlags)
	call IGetKey
       cp kCapA
       jr z,AboutScreen
       cp kMath
       jr z,AboutScreen
       cp kCapV
       jr z,ViewFile
       cp k6
       jr z,ViewFile
       cp kMode
       jr z,AppExit
	cp kQuit
	jr z,AppExit
	cp kClear
	jp z,AppExit
	cp kYequ
	jr z,Explore_Add
	cp kTrace
	jr z,Explore_New
       cp kLeft
       jr z,Explore_GoBack
;Above keys work all the time
	ld e,a
	ld bc,(file_count)
	ld a,b
	or c
	jr z,Explore_KeyLoop
	ld a,e
;Below keys work only when at least one file exists
       cp kSub
       jr z,Explore_PageUp
       cp kAdd
       jr z,Explore_PageDown
      	cp kDown
	jr z,Explore_KeyDown
	cp kUp
	jr z,Explore_KeyUp
	cp kGraph
	jr z,Explore_Ren
	cp kWindow
	jr z,Explore_Copy
	cp kZoom
	jr z,Explore_Del
       cp kDel
       jr z,Explore_Del
	cp kEnter
	jr z,Explore_KeyEnter
       ld a,(cur_entry+11)
       and 10h
       jr z,Explore_KeyLoop
       ld a,e
;Below keys work only when highlighted entry is directory
       cp kRight
       jr z,Explore_KeyEnter
	jr Explore_KeyLoop
;-----------------------------------------------------------------------------
Explore_PageUp:
       ld hl,(file_cur)
       ld a,h
       or l
       jr z,Explore_KeyLoop
       ld de,7
       sbc hl,de
       jr c,EPU_Weird
       ld (file_cur),hl
       ld hl,(file_start)
       sbc hl,de
       jr c,EPU_Weird2
       ld (file_start),hl
       jr Explore_LoopBack
EPU_Weird:
       ld hl,0
       ld (file_cur),hl
EPU_Weird2:
       ld hl,0
       ld (file_start),hl
       jr Explore_LoopBack
Explore_PageDown:
       ld hl,(file_cur)
       ld de,7
       add hl,de
       ex de,hl
       ld hl,(file_count)
       B_CALL cphlde
       jr c,EPD_End
       jr z,EPD_End
       sbc hl,de
       ld de,7
       B_CALL cphlde
       jr c,EPD_Weird
       ld hl,(file_start)
       add hl,de
       ld (file_start),hl
EPD_Normal:
       ld hl,(file_cur)
       add hl,de
       inc hl
EPD_End:
       dec hl
       ld (file_cur),hl
       jr Explore_LoopBack
EPD_Weird:
       ld hl,(file_count)
       or a
       sbc hl,de
       ld (file_start),hl
       jr EPD_Normal
;-----------------------------------------------------------------------------
AboutScreen:
       B_CALL ClrLCDFull
       ld hl,0
       ld (pencol),hl
       ld hl,sAbout
       ld b,3
AS_Loop_A:
       push bc
       call vputstring
       ld a,(penrow)
       add a,6
       ld (penrow),a
       xor a
       ld (pencol),a
       pop bc
       djnz AS_Loop_A
       ld hl,1900h
       ld (pencol),hl
       ld hl,sThanks
       ld b,4
AS_Loop:
       push bc
       call vputstring
       ld a,(penrow)
       add a,7
       ld (penrow),a
       xor a
       ld (pencol),a
       pop bc
       djnz AS_Loop
       ld hl,3900h
       ld (pencol),hl
       ld hl,sURL
       call vputstring
       call IGetKey
       jr Explore_ListDirectory
;-----------------------------------------------------------------------------
Explore_GoBack:
       ld a,(current_dir+1)
       or a
       jr z,Explore_KeyLoop
       jr Explore_GoUp
Explore_KeyEnter:
	;Enter was pressed.
	ld a,(cur_entry+11)
	and 10h
	jr z,Explore_KeyEnter_File
	;It's a directory. Check if it is "." or ".."
	ld a,(cur_entry)
	cp '.'
	jr nz,$F
	ld a,(cur_entry+1)
	cp '.'
	jr nz,$F
Explore_GoUp:
	;Move up a directory
	ld hl,current_dir
	xor a
	ld bc,128
	cpir
	ld a,'/'
	ld bc,128
	cpdr
	ld de,current_dir
	or a
	sbc hl,de
	jr c,Explore_KeyEnter_MoveUp
	add hl,de
	inc hl
	ld (hl),0
	jr Explore_ListDirectory
Explore_KeyEnter_MoveUp:
	xor a
	ld (current_dir+1),a
	jr Explore_ListDirectory
$$:    ld hl,current_dir
	xor a
	ld bc,128
	cpir
	dec hl
	dec hl
	ld a,(hl)
	inc hl
	cp '/'
	jr z,$F
	ld (hl),'/'
	inc hl
$$:	push hl
	ld de,OP1
	ld hl,cur_entry
	U_CALL FAT_nameConvertFrom11
	pop de
Explore_KeyEnter_1:
	ld a,(hl)
	or a
	jr z,$F
	ld (de),a
	inc hl
	inc de
	jr Explore_KeyEnter_1
$$:	xor a
	ld (de),a
	jp Explore_ListDirectory
Explore_KeyEnter_File:
;Enter pressed on a file. Do whatever here.
;Try and find built-in handle for this file type
       ld hl,bTypeList
ekef_loop:
       ld a,(hl)
       or a ;end of list?
       jr z,Explore_KeyLoop
       ld b,3
       ld de,cur_entry+8
       push hl
       call cmpstrb
       pop hl
       inc hl
       inc hl
       inc hl
       ld e,(hl)
       inc hl
       ld d,(hl)
       inc hl
       jr nz,ekef_loop
       ex de,hl
       jp (hl)
;-----------------------------------------------------------------------------
Explore_KeyUp:
	ld hl,(file_cur)
	ld (OP4),hl
	ld a,h
	or l
	jr z,Explore_KeyLoop
	dec hl
	ld (file_cur),hl
	ld de,(file_start)
	or a
	sbc hl,de
	jr nc,Explore_KeyDown_Rects
	dec de
	ld (file_start),de
	jr Explore_LoopBack
Explore_KeyDown:
	ld de,(file_cur)
	ld (OP4),de
	inc de
	ld hl,(file_count)
	or a
	sbc hl,de
	jr z,Explore_KeyLoop
	ld (file_cur),de
	ld hl,(file_start)
	ld bc,7
	add hl,bc
	ex de,hl
	or a
	sbc hl,de
	jr c,Explore_KeyDown_Rects
	ld de,(file_start)
	inc de
	ld (file_start),de
	jr Explore_LoopBack
Explore_KeyDown_Rects:
	;Turn off original black rectangle
	ld hl,(OP4)
	ld de,(file_start)
	or a
	sbc hl,de
	call Explore_Rects_R
	ld hl,(file_cur)
	ld de,(file_start)
	or a
	sbc hl,de
	call Explore_Rects_R
	jr Explore_LoopBack_NoList
Explore_Rects_R:
	ld a,l
	add a,a
	ld b,a
	add a,a
	add a,b
	add a,8
	ld h,a
	add a,5
	ld d,a
	ld l,0
	ld e,50
	B_CALL InvertRect
DummyRet:
	ret
;-----------------------------------------------------------------------------
Explore_DisplayFileSize:
	;Input - DEHL is a 32-bit number.
	;Output - Displays the file size.
	;Bytes for <= 64 KB
	;Kilobytes for <=16MB
	;Megabytes otherwise
	ld a,d
	or a
	jr z,$F
	;We have at least 16 MB. Divide by 2^20
	ld h,d
	ld l,e
	ld de,0
	srl h
	rr l
	srl h
	rr l
	srl h
	rr l
	srl h
	rr l
	ld a,'M'
	ld (OP6),a
	jr Explore_DFS_1
$$:
	;Check for over 64 KB?
	ld a,e
	or e
	jr z,$F
	;Divide by 2^10.
	ld l,h
	ld h,e
	ld e,0
	srl h
	rr l
	srl h
	rr l
	ld a,'K'
	ld (OP6),a
	jr Explore_DFS_1
$$:
Explore_DisplayByteSize:
	xor a
	ld (OP6),a
Explore_DFS_1:
	ld ix,OP5+10
	ld (ix+2),0
Explore_DFS_2:
	ld a,10
	call Divide32
	;A is remainder
	add a,'0'
	ld (ix),a
	dec ix
	ld a,d
	or e
	or h
	or l
	jr nz,Explore_DFS_2
	push ix
	pop hl
	inc hl
	jp vputstring
;-----------------------------------------------------------------------------
AppExit:
	U_CALL HostKill
	U_CALL DriverKill
AppExitNoDrv:
       im 1
	xor a
	out (20h),a
       B_CALL GrBufClr
       set 0,(iy+3)
       res 1,(iy+2)
       res 7,(iy+14h)
       xor a
       ld (iy+5),a
       ld (MenuCurrent),a
       set apdAble,(iy+apdFlags)
       B_CALL DelRes
       rst 38h
       B_CALL GetCSC
       ld hl,sHomescreen
       rst 20h
       B_CALL ChkFindSym
       B_CALL DelVar
       ld hl,0
       B_CALL CreateProg
       ld hl,cmdShadow
       ld (hl),' '
       ld de,cmdShadow+1
       ld bc,127
       ldir
       B_CALL ClrLCDFull
       B_CALL HomeUp
       ld hl,0
       ld (cmdshadcur),hl
       ld (cmdcursor),hl
       ld a,kClear
	B_JUMP JForceCmd
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
sAbout:
       DB "msd8x v0.95",0
       DB "Brandon Wilson",0
       DB "Michael Vincent",0
sThanks:
       DB "Thanks to:",0
       DB "Dan Englender",0
       DB "Kevin Frew",0
       DB "And many others...",0
sURL:
       DB "http://msd8x.denglend.net",0
;-----------------------------------------------------------------------------
sReconnect:
       DB "Disconnect and  "
       DB "reconnect your  "
       DB "mass storage    "
       DB "device, then    "
       DB "press any key.",0
sIndexing:
	DB "INDEXING",0CEh,0
sSorting:
	DB "SORTING",0CEh,0
sOK:
	DB "OK",0
sSize:
	DB "Size:",6,6,6,6,6,6,0
sAttributes:
	DB "Attr:",6,6,6,6,6,6,0
sNavTable:
	DB " Add",0
	DB " Copy",0
	DB "   Del",0
	DB "  New",0
	DB "  Ren",0
Explore_Attrs:
	DB "RHS",0,0,"A"
sRoot:
	DB "/",0
sHomescreen:
       DB 05h,21h,0
;-----------------------------------------------------------------------------
;To add new handles for file types, add the extension and a label to the
; code to handle it. Make sure zero remains the last byte in the table.
;
;current_dir contains ASCIIZ directory path.
;cur_entry contains 11-character name of selected file.
;saveSScreen and cmdShadow are free for use.
;temp_handle is a convenient spot for a file handle.
;temp_file is a convenient spot to store a file/directory path.
;There's no need to init the mass storage driver; you can use FAT_/DOS_ calls
; immediately.
;Do not make changes to statVars.
;If you alter the directory contents, end execution by jumping to
; Explore_ListDirectory. Otherwise, use Explore_ListDirectoryStay.
;If you need to kill the mass storage driver (to use another, for example),
; use HostKill and/or DriverKill. Let msd8x re-initialize itself by jumping
; to InitProg when you're done.
;
;See images.asm, runprog.asm, and txtview.asm for examples.
bTypeList:
       DB "8XP"
       DW Explore_RunProg
       DB "8XI"
       DW Explore_Disp8XI
       DB "TXT"
       DW ViewFile
       DB 0
